\ ==============================================================
\ [TITLE]     8052-ANS-Forth SERVO52 woorden. 
\ [FILE]      SERVO52.FRT
\ [CATEGORY]  Tools
\ [VSN]       1.00 
\ [CREATED]   April 4, 1996, Willem Ouwerkerk 
\ [LASTCHNGD] August 16, 1998, Willem Ouwerkerk 
\ [COPYRIGHT] Dutch Forth Users Group (C) 1996-1998
\ [MODULE OF] 8052-ANS-Forth 
\ [AUTHOR]    Willem Ouwerkerk 
\ [PURPOSE]   Controling remote control model servo's
\ ==============================================================

?DEF -SERVO  [IF]  -SERVO  [THEN]  MARKER -SERVO

#20000          CONSTANT REITERATE      \ Pulse repeat time 20 ms
#1000           CONSTANT MINIMUM        \ Minimum pulse lenght 1 ms

$37             SFR STAND               \ State of servo interrupt
$38             SFR SERVO1-HIGH         \ Pulse length servo 1
$39             SFR SERVO1-LOW 
$3A             SFR SERVO2-HIGH         \ Pulse length servo 2
$3B             SFR SERVO2-LOW 
$3C             SFR PAUSE-HIGH          \ Pause is REITERATE-(SERVO1+SERVO2)
$3D             SFR PAUSE-LOW 

$89             SFR TMOD                \ 8052 special function registers
$8C             SFR TH0 
$8A             SFR TL0
$A9             BIT ET0                 \ 8052 bit i/o
$8C             BIT TR0    
$AF             BIT EAL 


DOC  Function explanation

Most servo's are controlled with a positive pulse from 1 to 
2 ms from one limit to the other. Only one bit is needed for 
each servo. The mechanism used in this code, can be extended 
easy with extra servo outputs, because all code blocks have 
the same structure.

STAND     0 = Set timing of servo 1 pulse.
          1 = Set timing of servo 2 pulse.
          2 = Set timing of pause pulse.

P1.0        = Servo output 1
P1.1        = Servo output 2

This implementation keepes the repeat time constant, so the
an osciloscoop picture will be solid on its screen.

For a short and simple implementation the PAUSE value may be
a constant value of -20000 at 12 MHz for a repeat time of 
20 millisec. The words CALCULATE PULSE1 and PULSE2 may be left 
out then and CONVERT may be written out directly in SERVO1 and 
SERVO2 or a subword of them.

The words .SERVO1 .SERVO2 and .PAUSE may be left out also, they
are there only for showing the inner workings of the code.

ENDDOC

: PULSE1        ( -- n )                \ Give servo one pulse
    SERVO1-HIGH ><  SERVO1-LOW  OR
    ;

: PULSE2        ( -- n )
    SERVO2-HIGH ><  SERVO2-LOW  OR      \ Give servo two pulse
    ;

: CONVERT       ( n1 -- n2 )            \ Calculate real timer 0 value
    NEGATE                              \ For 12 MHz xtal 
\   2*  NEGATE                          \ 2* for 24 MHz xtal
    ;

: CALCULATE     ( -- )                  \ Calculate actual pause time
    REITERATE CONVERT                   \ Convert basic pause time
    PULSE1 -  PULSE2 -                  \ Leave actual pause time
    CLEAR ET0                           \ Timer int. off
    SPLIT TO PAUSE-HIGH  TO PAUSE-LOW   \ Set pause time
    SET ET0                             \ Timer int. on
    ; 

\G Set servo one to position +n.
: SERVO1        ( +n -- )               \ Servo52  Set servo 1
    MINIMUM UMIN  MINIMUM +             \ Calculate pulse width
    CONVERT                             \ Convert to timer value
    SPLIT  TO SERVO1-HIGH               \ Store in byte registers
    TO SERVO1-LOW  
    CALCULATE                           \ Calculate pause value
    ;

\G Set servo two to position +n.
: SERVO2        ( +n -- )               \ Servo52  Set servo 2
    MINIMUM UMIN  MINIMUM +             \ Calculate pulse width
    CONVERT                             \ Convert to timer value
    SPLIT  TO SERVO2-HIGH               \ Store in byte registers
    TO SERVO2-LOW  
    CALCULATE                           \ Calculate pause value
    ;

: .SERVO1   PULSE1  .HEX ;              \ To give some insight
: .SERVO2   PULSE2  .HEX ;
: .PAUSE    PAUSE-HIGH ><   PAUSE-LOW   OR .HEX ; 

\ Real interrupt routine for timer 0

LABEL INTERRUPT     ( -- addr )         \ Handle interrupt for 2 servo's
    ACC: PUSH,
    A: ADR STAND MOV,                   \ Get interrupt state
    ZER IF,                             \ Is it zero ?
        P1: .0 SETB,                    \ Servo two high
        ADR TH0  ADR SERVO1-HIGH MOV,   \ Yes, reload timer 0 with SERVO1 pulse
        ADR TL0  ADR SERVO1-LOW MOV,
        ADR STAND INC,                  \ Next state is one
        ACC: POP,
        RETI,
    THEN,
    A: DEC,                             \ Decrease interrupt state
    ZER IF,                             \ Interrupt state zero ?
        P1: .0 CLR,                     \ Yes, then it was one, servo one low.
        P1: .1 SETB,                    \ Servo one high.
        ADR TH0  ADR SERVO2-HIGH MOV,   \ Reload timer 0 with SERVO2 pulse
        ADR TL0  ADR SERVO2-LOW MOV,
        ADR STAND INC,                  \ Next state is two
        ACC: POP,
        RETI,
    THEN,                               \ Interrupt state two.
    P1: .1 CLR,                         \ Servo two low.
    ADR TH0  ADR PAUSE-HIGH MOV,        \ Reload timer 0 with PAUSE pulse
    ADR TL0  ADR PAUSE-LOW MOV,
    ADR STAND 0 # MOV,                  \ Next state is zero
    ACC: POP,
    RETI,
END-CODE

: START-SERVO'S     ( -- )
    INTERRUPT  TF0-VEC SET-VECTOR       \ Install servo interrupt handler
    CLEAR STAND                         \ Always start at state 0
    TMOD  $F0 AND  1 OR  TO TMOD        \ Set Timer 0 mode 1 bit.
    -1  TO TH0                          \ Set negated 16 bits count,
    -1  TO TL0                          \ to start the servo interrupt
    SET EAL                             \ Global enable interrupts
    SET ET0                             \ Enable timer 0 interrupts
    SET TR0                             \ Start timer 0
    ;

: STOP-SERVO'S      ( -- )              \ Stop timer 0 and interrupt
    CLEAR ET0                           \ Disable timer 0 interrupts
    CLEAR TR0                           \ Stop timer 0
    TF0-VEC  CLEAR-VECTOR               \ Remove servo interrupt handler
    ;
    

\ Some examples

500 SERVO1  500 SERVO2                  \ Set basic servo position

145 TO FUDGE                            \ 24 MHz xtal

: TEST1         ( +n -- )               \ Let servo's step
    START-SERVO'S
    LOCALS| WACHT |
    BEGIN
        0 SERVO1  1000 SERVO2
        WACHT MS                        \ Wait a moment
        500 SERVO1  500 SERVO2
        WACHT MS                        \ Wait a moment
        1000 SERVO1  0 SERVO2
        WACHT MS                        \ Wait a moment
    KEY? UNTIL
    STOP-SERVO'S
    ;

: TEST2         ( +n -- )               \ Let servo's swing
    START-SERVO'S
    LOCAL WAIT                          \ Save wait time
    BEGIN
        MINIMUM 0 DO
            I SERVO1                    \ Servo's moving opposite
            MINIMUM I - SERVO2
            WAIT MS                     \ Wait a moment
        LOOP
        MINIMUM 0 DO
            I SERVO2                    \ Servo's moving opposite
            MINIMUM I - SERVO1
            WAIT MS                     \ Wait a moment
        LOOP
    KEY? UNTIL
    STOP-SERVO'S
    ;

: DEMO1     333 TEST1 ;                 \ Usefull test
: DEMO2       2 TEST2 ;                 \ Usefull test

\ End

