\ ==============================================================
\ [TITLE]     8051 ANS-Forth CLOCK word set. 
\ [FILE]      CLOCK.FRT
\ [CATEGORY]  Example
\ [VSN]       1.00 
\ [CREATED]   July 17, 1994, Willem Ouwerkerk 
\ [LASTCHNGD] October 18, 1994, Willem Ouwerkerk 
\ [COPYRIGHT] Dutch Forth Users Group (C) 1994
\ [COMPILE]   CursusForth Environments
\ [MODULE OF] 8051-ANS-Forth 
\ [USES]      ...
\ [AUTHOR]    Willem Ouwerkerk
\ [PURPOSE]   Example of timer use, interupt use and SFR use.
\             The result is a 24 hour clock.
\ ==============================================================

MARKER -CLOCK                           \ Timer interupt test

ROM

INTERNAL ALSO  DEFINITIONS

#-10000 CONSTANT PERIOD                 \ Ticks interval period

    $34 SFR TICKS                       \ Peephole to clock
    $35 SFR SEC  
    $36 SFR MINU  
    $37 SFR HOUR 

FORTH DEFINITIONS

\G Set the 24 hour clock to the time given on the stack.
: SET-TIME      ( hour min sec -- )     \ Clock  Set new time
    TO SEC
    TO MINU
    TO HOUR
    ;

\G Get current time from the 24 hour clock to the stack.
: TIME          ( -- sec min hour )     \ Clock  Push current time
    SEC  MINU  HOUR
    ;

INTERNAL DEFINITIONS

: (.)           ( 0 .. 99 -- )          \ Print two decimal digits
    0 <# # # #> TYPE
    ;

: (.TIME)       ( ticks sec min hour -- )   \ Print clock in format
    (.)  ." :"
    (.)  ." :"
    (.)  ." ."
    (.)  SPACE
    ;

FORTH DEFINITIONS

\G Print current time in the format: hh:mm:ss.xx .
: .TIME         ( -- )                  \ Clock  "dot-time" Print current time
    TICKS  TIME  (.TIME)                \ Print hours, mins, secs and ticks
    ;

\ A timer function to measure execution time etc.

INTERNAL DEFINITIONS

0 VALUE 'TICKS                          \ Remember current time
0 VALUE 'SEC
0 VALUE 'MIN
0 VALUE 'HOUR

FORTH DEFINITIONS

\G Start the timer, see also TIMER and .TIMER
: RESET-TIMER   ( -- )                  \ Clock  Save current time
    TIME    TO 'HOUR
            TO 'MIN
            TO 'SEC
    TICKS   TO 'TICKS
    ;

\G Give elapsed time on the stack since the last execution of 
\G RESET-TIMER . See also: .TIMER
: TIMER         ( -- Dticks Dsec Dmin Dhour ) \ Clock 
    TICKS   'TICKS  -                   \ Leave difference
    SEC     'SEC    -
    MINU    'MIN    -
    HOUR    'HOUR   -
    ;

\G Print elapsed time since last execution of RESET-TIMER in
\G format: hh:mm:ss.xx . See also: TIMER
: .TIMER            ( -- )              \ Clock "dot-timer"
    TIMER   (.TIME)
    ;

ASSEMBLER DEFINITIONS

    $89 CONSTANT TMOD:                  \ 80C535 assembler labels
    $8C CONSTANT TH0:
    $8A CONSTANT TL0:   
    $A9 CONSTANT ET0:
    $8C CONSTANT TR0:   
    $AF CONSTANT EAL:
    $34 CONSTANT =TICKS
    $35 CONSTANT =SEC            
    $36 CONSTANT =MINU            
    $37 CONSTANT =HOUR           

FORTH DEFINITIONS

\G Start timer 0 and enable timer interupts.
CODE START-T0       ( -- )              \ Clock "start-timer-0" Start timer 0 and interupt
    TMOD: $F0 # ANL,                    \ Clear Timer 0 mode bits
    TMOD: $01 # ORL,                    \ Set Timer 0 mode 1
    TH0: PERIOD >< # MOV,               \ Set negated 16 bits count
    TL0: PERIOD # MOV,
    =TICKS 0 # MOV,                     \ Initialise ticks to zero
    EAL: SETB,                          \ Global enable interupts
    ET0: SETB,                          \ Enable timer 0 interrupts
    TR0: SETB,                          \ Start timer 0
    RET,                                \ Done
END-CODE

\G Stop timer 0 and disable timer interupts
CODE STOP-T0        ( -- )              \ Clock "stop-timer-0" Stop timer 0 and interupt
    EAL: CLR,                           \ Global enable interupts
    ET0: CLR,                           \ Disable timer 0 interrupts
    TR0: CLR,                           \ Stop timer 0
    RET,
END-CODE

\G Clock timer interupt routine. It increases every one hundredth
\G of a second the tick timer, building a real time clock.
LABEL CLOCK-INT     ( -- addr )         \ Clock "clock-interupt" Executed every PERIOD ticks
    ACC: PUSH,                          \ Save used registers
    PSW: PUSH,
    TH0: PERIOD >< # MOV,               \ Set negated 16 bit count
    TL0: PERIOD # MOV,

    =TICKS INC,                         \ Increase ticks counter
    A: =TICKS MOV,                      \ Get ticks for comparision
    A: #100 # EQL IF,                   \ Ticks counted one second ?
        =TICKS 0 # MOV,

        =SEC INC,
        A: =SEC MOV,
        A: #60 # EQL IF,                \ Seconds counted one minute ?
            =SEC 0 # MOV,

            =MINU INC,
            A: =MINU MOV,
            A: #60 # EQL IF,            \ Minutes counted one hour ?
                =MINU 0 # MOV,

                =HOUR INC,
                A: =HOUR MOV,
                A: #24 # EQL IF,        \ Hours counted 24 hour ?
                    =HOUR 0 # MOV,
                THEN,
            THEN,
        THEN,
    THEN,
    PSW: POP,                           \ Restore used registers
    ACC: POP,    
    RETI,                               \ Return
END-CODE

: BOOT      ( -- )                      \ Allow timer to run !!
    START-T0  BOOT
    ;

CLOCK-INT  TF0-VEC SET-VECTOR           \ Install timer 0 interupt

\ ' BOOT AUTOSTART                      \ Remember this 

PREVIOUS  FORTH

                    \ (* End of Source *) /
