\ ==============================================================
\ [TITLE]     8052-ANS-Forth I2C driver for 8574, 8591 and 8584. 
\ [FILE]      I2C-DEMO.FRT
\ [CATEGORY]  Example
\ [VSN]       1.20 
\ [CREATED]   December 11, 1995, Willem Ouwerkerk 
\ [LASTCHNGD] October 13, 1998, Willem Ouwerkerk 
\ [COPYRIGHT] De Elektrise Schaar (C) 1995-1998
\ [USES]      ...
\ [PURPOSE]   Use of I2C primitives to implement specific drivers
\             and use of locals. See: TEST-ALARM  !!!!
\ ==============================================================

\ Read and write to ADC/DAC on a PCF8591 at address zero.

$90     CONSTANT ADC-ID                 \ I2C-demo  identification address

\G This flag is set after the DAC is used. Clear it if
\G the DAC must be off during ADC conversions.
FLAG DAC?    ( -- vlag )                \ I2C-demo  Keep DAC active if true

\G Read analog input +n of a PCF8591, return the result u .
: ADC           ( +n -- u )             \ I2C-demo  Read input +n from adc
    START-BIT
    ADC-ID BYTE-OUT                     \ Send chip write address
    3 AND  DAC? $40 AND  OR BYTE-UIT    \ Select input and DAC active bit
    START-BIT                           \ Give 'repeated start' 
    ADC-ID 1+ BYTE-OUT                  \ Send chip read address
    BYTE-IN  ACK-BIT  DROP              \ Throw away old conversion
    BYTE-IN  NACK-BIT                   \ Read fresh conversion
    STOP-BIT                            \ Ready
    ;

\G Set the analog output of a PCF8591 to the value u.
: DAC           ( u -- )                \ I2C-demo  Send u to dac
    SET DAC?                            \ DAC active
    START-BIT
    ADC-ID BYTE-OUT                     \ Send chip write address
    $40 BYTE-OUT                        \ Send control and
    BYTE-OUT                            \ databyte
    STOP-BIT                            \ Ready
    ;

\G Show sawtooth waveform at DAC output with interval +n millisecs
\G until a key is hit.
: WAVE          ( +n -- )               \ I2C-demo  Sawtooth at dac
    LOCAL GAP                           \ Remember interval
    BEGIN
        256 0 DO                        \ Make sawtooth
            I DAC  GAP MS               \ Set DAC and wait
        LOOP
    KEY? UNTIL
    ;

\ End PCF8591
\ PCF8583, 24 hours clock
\
\ 20 30 10    SET-CLOCK  is: 10 hours 30 min. 20 sec. 0 ticks
\ 29 01 1998  SET-DATE   is: 29-01-1998
\ The 1/100 sec. clock/alarm register is always set to zero here.

\ PCF8583 primitives

$A0 CONSTANT CLOCK-ID                   \ PCF8583 identification address

\G Store x in the clock or ram registers addr of a PCF8583.
: !CLOCK        ( x addr -- )           \ I2C-demo "store-clock"  Store x at address addr
    START-BIT
    CLOCK-ID BYTE-OUT                   \ Send chip read address
    BYTE-OUT                            \ Send address
    BYTE-OUT                            \ and databyte
    STOP-BIT                            \ Ready
    ;

\G Read x from the clock or ram registers addr of a PCF8583.
: @CLOCK        ( addr -- x )           \ I2C-demo "fetch-clock"  Read de inhoud van addr, x
    START-BIT
    CLOCK-ID BYTE-OUT                   \ Send chip write address
    BYTE-OUT                            \ Send address
    START-BIT                           \ Give 'repeated start' 
    CLOCK-ID 1+ BYTE-OUT                \ Send chip read address
    BYTE-IN                             \ Read contents of address 
    NACK-BIT
    STOP-BIT                            \ Ready
    ;

\ PCF8583 BCD conversion

: >BCD              ( byte1 -- byte2 )  \ "to-bcd" Convert binary to BCD.
    #10 /MOD  #16 *  +                  \ Make BCD number
    ;

: BCD>              ( byte1 -- byte2 )  \ "from-bcd" Convert BCD to binary.
    #16 /MOD  #10 *  +                  \ Make binary number
    ;

\ PCF8583 clock 

: STOP-CLOCK        ( -- )              \ Stop clock functions
    0 @CLOCK  $80 OR  0 !CLOCK
    ;

: START-CLOCK       ( -- )
    0 @CLOCK  $7F AND  0 !CLOCK         \ Start clock functions
    ;

$00 CONSTANT NORMAL                     \ I2C-demo  Clock without alarm
$04 CONSTANT WAKEUP                     \ I2C-demo  Clock with alarm

\G Set clock mode of a PCF8583 chip, mode can be NORMAL or WAKEUP.
: CLOCK             ( mode -- )         \ I2C-demo  Set clock mode for PCF8583
    0 !CLOCK
    ;

\G Set time to hour min sec ticks.
: SET-TIME  ( sec min hour -- )         \ I2C-demo  Set PCF8583 time
    STOP-CLOCK
    0 1 !CLOCK  >BCD 4 !CLOCK           \ Convert to BCD and store
    >BCD 3 !CLOCK  >BCD 2 !CLOCK
    START-CLOCK
    ;

\G Read time, return sec min hour.
: GET-TIME  ( -- sec min hour )         \ I2C-demo  Read time from PCF8583
    2 @CLOCK BCD>  3 @CLOCK BCD>        \ and set to binary
    4 @CLOCK BCD>
    ;

\G Print current time
: .TIME             ( -- )              \ Print time
    GET-TIME . . . 
    ;

VARIABLE YEAR                           \ Year counter
VARIABLE #YEAR                          \ Reserve year control

\G Set date to day month year.
: SET-DATE  ( day month year -- )       \ I2C-demo  Set date
    STOP-CLOCK
    DUP TO YEAR  DUP 3 AND TO #YEAR     \ Set year
    6 LSHIFT >R  >BCD 6 !CLOCK          \ Scale year bits and store month
    >BCD R> OR  5 !CLOCK                \ Pack day with year and store
    START-CLOCK
    ;

\G Read date, return day month year.
: GET-DATE  ( --  day month year )      \ I2C-demo  Get date
    5 @CLOCK 6 RSHIFT  DUP FROM #YEAR <> \ New year started ?
    IF  1 +TO YEAR  THEN  TO #YEAR      \ Yes, increase year counter
    5 @CLOCK $1F AND BCD>               \ Get day
    6 @CLOCK $3F AND BCD>  FROM YEAR    \ Get month & year
    ;

\ Use alarm clock function of PCF8583

$00 CONSTANT NO                         \ I2C-demo  Alarm off
$90 CONSTANT DAY                        \ I2C-demo  Alarm every day at .. hour
$A0 CONSTANT WEEKDAY                    \ I2C-demo  Alarm every .. weekday at ..
$B0 CONSTANT DATE                       \ I2C-demo  Alarm date at .. 

\G Set alarm mode of an PCF8583 chip. Valid alarm modes 
\G are: DAY  WEEKDAY  DATE  NO
: ALARM             ( mode -- )         \ I2C-demo  Select alarm mode
    8 !CLOCK
    ;

\G Set alarm time to sec. min. hour.
: SET-ALARM-TIME    ( sec min hour -- ) \ I2C-demo  Set alarm time and mode
    0 9 !CLOCK  >BCD 12 !CLOCK          \ Ticks always zero, set sec.
    >BCD 11 !CLOCK  >BCD 10 !CLOCK      \ Set min. and hours
    ;

\G Read alarm time, return sec min hour.
: GET-ALARM-TIME    ( -- sec min hour ) \ I2C-demo  Read alarm time
    10 @CLOCK BCD>  11 @CLOCK BCD>      \ Read hours, minutes
    12 @CLOCK BCD>                      \ and seconds
    ;

\G Print alarm time.
: .ALARM-TIME       ( -- )              \ Print alarm time
    GET-ALARM-TIME . . .
    ;

\G Set alarm date to day month year. Note: The year bits are not used.
: SET-ALARM-DATE    ( day month year -- ) \ I2C-demo  Set day, month and year
    6 LSHIFT >R  >BCD 14 !CLOCK
    >BCD R> OR  13 !CLOCK
    ;

\G Set alarm weekday (bits 0 to 6).
: SET-ALARM-WEEKDAY ( x -- )            \ I2C-demo  Each bit is a weekday
    14 !CLOCK
    ;

\G Read alarm flag, return true if the alarm went off, otherwise false.
: ALARM?            ( -- flag )         \ I2C-demo  Alarm went of ?
      0 @CLOCK  2 AND 0<>
      ;

\G Test alarm clock function, +n is the interval in seconds between
\G the alarm intervals (+n is max. 59). Hit any key to leave the test.
: TEST-ALARM        ( +n -- )           \ I2C-demo test (alarm)clock
    WAKEUP CLOCK  DAY ALARM             \ Set alarm mode
    59 UMIN  LOCAL GAP                  \ Maximum test interval in seconds
    BEGIN
        WAKEUP CLOCK                    \ Initialise alarm clock
        GET-TIME  LOCALS| HR MIN SEC |
        GAP +TO SEC  SEC 59 U> IF       \ Make new alarm time, Min. overflow ?
            -60 +TO SEC                 \ Correct seconds
            1 +TO MIN  MIN 60 = IF      \ Correct minutes, min. overflow ?
                0 TO MIN  1 +TO HR      \ Correct minutes and hour
                HR 24 = IF  0 TO HR  THEN
            THEN
        THEN
        SEC MIN HR  SET-ALARM-TIME      \ Set alarm time
        CR ." Time: " .TIME             \ Show (alarm)time
        ."  Next alarm at: " .ALARM-TIME
        BEGIN  ALARM? UNTIL             \ Wait for alarm to go off
        7 EMIT  CR ." Alarm at: " .TIME \ Ring bell and show time
    KEY? UNTIL
    ;

\G Show contents of all clock and alarm registers.
: DUMP-CLOCK        ( -- )              \ Show all clock registers
    16 0 DO
        I @CLOCK  I PAD + C!
    LOOP
    PAD 16 DUMP
    ;

\G Initialise a normal clock and reset the alarm.
: SETUP-CLOCK       ( -- )              \ I2C-demo  
    NORMAL CLOCK  NO ALARM              \ Set clock on and alarm off
    ;

: BOOT-I2C          ( -- )              \ I2C-demo  Initialise and show demo
    SETUP-I2C  SETUP-CLOCK  PAGE
    ." Test clock and alarm clock" 
    CR 10 TEST-ALARM  1000 MS           \ Give alarm every 10 seconds
    ;

' BOOT-I2C      AUTOSTART               \ Initialise and secure I2C routines

\ End
