\ ==============================================================
\ [TITLE]     8052-ANS-Forth single master I2C primitives. 
\ [FILE]      I2C.FRT
\ [CATEGORY]  Tool
\ [VSN]       1.20 
\ [CREATED]   December 11, 1995, Willem Ouwerkerk 
\ [LASTCHNGD] August 19, 1998, Willem Ouwerkerk 
\ [COPYRIGHT] De Elektrise Schaar (C) 1995-1998
\ [USES]      ...
\ [PURPOSE]   Use of high level bit I/O to implement single
\             master I2C routines and basic PCF8574(a) I/O.
\ ==============================================================
\ Parts:        2 resistors.

\ P3.4 = SDA = I2C data line
\ P3.5 = SCL = I2C clock line
\
\ All primitive routines in this version are ending, setting
\ the SCL line low except for STOP-BIT !!!! Clock stretching
\ is not supported yet, see the example routine WAIT-CLOCK .
\
\ After an error condition the SCL and SDA line are left high,
\ unless some connected device pulls these lines low !!
\
\ Note: When xtal is not 12 MHz, the word WAIT must be adjusted.

?DEF -I2C   [IF]  -I2C  [THEN]  MARKER -I2C

VOCABULARY I2C  I2C ALSO  DEFINITIONS

ASSEMBLER                               \ Prototype for 8574, 8583 and 8591
P3: .4 BIT-SFR SCL                      \ Serial Clock line
P3: .5 BIT-SFR SDA                      \ Serial Data line
FORTH

\ I2C primitive routines

: WAIT          ( -- )                  \ Wait 5 microsec.
    NOOP
    ;

DOC  Clockbit stretching

\ This routine performs synchronization between a fast clock and
\ a slow device. For more detail, see par 6.1, 6.3 and par. 10.0 
\ of 'The I2C-bus specification'. This routine was not needed for 
\ the chips on the tested breadboard (8574, 8583 and 8591).

CODE WAIT-CLOCK ( -- )                  \ Wait 7 microsec. or until the
    R2: 0 # MOV,                        \ Clock (SCL) timeout after 1 us
    BEGIN,                              \ 1020 us, is approx: 1 millisec.
    SCL BS WHILE,                       \ Wait until SCL is free. 2 us
        NOP,                            \ 1 us
    R2: DZER UNTIL,                     \ 2 us
    ] 3 THROW [                         \ Timeout on clock
    THEN,
    RET,                                \ 2 + 2 us (ACALL)
END-CODE

ENDDOC

: SETUP-I2C      ( -- )                 \ Setup I2C I/O bits
    SET SDA  SET SCL
    ;

\G Generate I2C start bit.
: START-BIT     ( -- )                  \ I2C  Generate startbit
    SET SDA  SET SCL  CLEAR SDA  
    WAIT  CLEAR SCL
    ;

\G Generate I2C stop bit.
: STOP-BIT      ( -- )                  \ I2C  Generate stopbit
    CLEAR SDA  SET SCL  WAIT  SET SDA
    ;

\G Generate negative acknowledge bit.
: NACK-BIT      ( -- )                  \ I2C  Generate no-ackbit
    SET SDA  SET SCL  WAIT  CLEAR SCL
    ;

\G Generate acknowledge bit.
: ACK-BIT       ( -- )                  \ I2C  Generate ackbit
    CLEAR SDA  SET SCL  WAIT  CLEAR SCL  SET SDA
    ;

\G Leave true if an acknowledge bit is detected, otherwise false.
: ACK?          ( -- flag )             \ I2C  Test incoming ackbit, flag is
    SET SDA  SET SCL  WAIT 
    SDA 0=  CLEAR SCL                   \ true if an ack is received
    ;

\G Send one byte to an I2C device. This routine does noet wait for 
\G an acknowledge.
: (BYTE-OUT)    ( byte -- )             \ I2C  Send byte
    8 0 DO
        DUP 128 AND  TO SDA  2*
        SET SCL  WAIT  CLEAR SCL  WAIT
    LOOP
    DROP
    ;

\G Send one byte to an I2C device. This routine waits for an 
\G acknowledge. If no acknowledge is received it generates a 
\G stop condition.
: BYTE-OUT      ( byte -- )             \ I2C  Send byte and test ack
    (BYTE-OUT)  ACK? 0= IF  STOP-BIT  THEN
    ;

\G Receive one byte from an I2C device, do not respond with an
\G (negative)acknowledge.
: BYTE-IN       ( -- byte )             \ I2C  Receive byte
    0  8 0 DO
        WAIT  SET SCL  2*  SDA 1 AND  OR  
        CLEAR SCL
    LOOP
    ;

\G Leave true if the chip dev-id is present, otherwise false.
: CHIP-PRESENT? ( dev-id -- flag )      \ I2C  Give true if chip dev-id is present
    START-BIT  (BYTE-OUT)  ACK?  STOP-BIT
    ;

\ Input and output routine for PCF8574(a) chips.

$40 CONSTANT I/O-ID1                    \ PCF8574 identification address
$70 CONSTANT I/O-ID2                    \ PCF8574a identification address

\G Set x at the outputs of the PCF8574 which responds to dev-id.
: !BYTE         ( x dev-id -- )         \ I2C "store-byte"
    START-BIT
    BYTE-OUT                            \ Send write address and
    BYTE-OUT                            \ databyte to chip
    STOP-BIT                            \ Ready
    ;

\G Read x from the inputs of the PCF8574 which responds to dev-id.
: @BYTE         ( dev-id -- x )         \ I2C "fetch-byte"
    START-BIT
    1+ BYTE-OUT                         \ Send read address to chip, than
    BYTE-IN                             \ read databyte from inputs
    NACK-BIT
    STOP-BIT
    ;

\ End
