\ ==============================================================
\ [TITLE]     8051 ANS-Forth optimiser words set. 
\ [FILE]      OPTIMISE.FRT
\ [CATEGORY]  Example
\ [VSN]       2.10 
\ [CREATED]   May 13, 1994, Willem Ouwerkerk 
\ [LASTCHNGD] October 19, 1994, Willem Ouwerkerk 
\ [COPYRIGHT] Dutch Forth Users Group (C) 1994
\ [COMPILE]   CursusForth Environments
\ [MODULE OF] Optimising compiler
\ [USES]      8051-ANS-Forth internal knowledge
\ [AUTHOR]    Willem Ouwerkerk 
\ [PURPOSE]   Optimizing of treathed code. Each word does it's
\             own optimising by inspecting a table with possible
\             combinations. Behind the combination is a type byte.
\             Thereafter is the <xt> of the replacement routine
\             in case of type (0). In case of type (1) there is
\             the <xt> of an optimiser macro.
\ ==============================================================

\G            Optimizing of treathed code. Each word does it's
\G            own optimising by inspecting a table with possible
\G            combinations. Behind the combination is a type byte.
\G            Thereafter is the <xt> of the replacement routine
\G            in case of type (0). In case of type (1) there is
\G            the <xt> of an optimiser macro.
\G            Two examples:
\G
\G            comp. $pointer  | type | optimiser or macro token
\G            --------------------------------------------------
\G For DROP   xt orig. DROP   |
\G            $adr of DROP    |  0   | xt of 2DROP
\G            $adr of 2DROP   |  0   | xt of 3DROP
\G            $adr of SWAP    |  0   | xt of NIP  
\G            $adr of DODOES  |  0   | xt of RDROP
\G            $adr of R>      |  1   | xt of macro SP: DEC, SP: DEC,
\G            -1              |

MARKER -OPTI                            \ Mark optimise file for testing

\ ---- Routines added to make code transferable ----------------
\ ---- For 8051 STC system -------------------------------------
\  For direct and indirect treathed systems this is probably @ and !

INTERNAL ALSO

\G Compile the execution token xt at addr.
: COMPILE!      ( xt addr -- )          \ Optimiser  Store token in memory
    1+ !                                \ Just after a call or jump
    ;

\ ---- Optimiser two, replace a code by other code or a call ---

\G Assemble the string ccc in the dictionary. Store the code length
\G in front of the assembled code. Leave its begin address c-addr
\G on the stack.
: ASS"              ( "ccc" -- c-addr ) \ Optimiser "assembler-string"
    ALSO  ASSEMBLER                     \ Assembler in search order
    HERE  0 C,                          \ Start of code string
    [CHAR] " PARSE EVALUATE             \ Parse and assemble a code string
    HERE OVER - 1-  OVER C!             \ Determine length and store it
    ALIGN  PREVIOUS                     \ Align string and remove assembler
    ;

\G Compile the string ccc in the dictionary. Store the code length
\G in front of the compiled code. Leave the begin address c-addr
\G on the stack.
: COLON"            ( "ccc" -- c-addr ) \ Optimiser "colon-string"
    HERE  0 C,  ]  RESET-OPTI           \ Start of code string, start compiling
    [CHAR] " PARSE EVALUATE             \ Parse and assemble a code string
    POSTPONE [                          \ Back to interpreter
    HERE OVER - 1-  OVER C!             \ Determine length and store it
    ALIGN                               \ Align string
    ;

\G ( C: -- macro-sys ) ( S: -- xt )
\G Create an execution token xt, enter compilation state and start
\G current macro definition, producing macro-sys.
\G
\G xt Execution: ( i*x -- j*x )
\G Execute the macro definition specified by xt. The stack effects
\G i*x and j*x represent arguments to and results from xt.
: :MACRO                                \ Optimiser "colon-macro" Build macro without a name
    :NONAME  ALSO ASSEMBLER             \ Search also assembler
    ;

\G End the current macro definition.
: ENDM      ( macro-sys -- )            \ Optimiser "end-macro" Close macro definition
    PREVIOUS  POSTPONE ;                \ Remove assembler from search order
    ;  IMMEDIATE                        \ It's immediate

INTERNAL DEFINITIONS
  
: COMP-ASS?         ( c-addr -- flag )
    COUNT 0                             \ Get length of wanted code string
    ?DO     C@+  OPT1 I + C@  <>        \ Test two code bytes, not equal ?
            IF      DROP FALSE          \ Yes, code not equal leave false
                    UNLOOP EXIT         \ and comparision loop.
            THEN
    LOOP    DROP TRUE                   \ Code strings equal, leave true.
    ;

: OPTIMISE,         ( table-addr -- )
    STATE @ 0=                          \ Are we executing *WO*
    IF      @ EXECUTE  EXIT             \ Yes, exec. original token
    THEN                                \ No, search table
\   ." OPT 0/1 " OPT0 .HEX  OPT1 .HEX   \ Debugging info
    >R  R@ CELL+                        \ Save address orig. token
    BEGIN   
        @+  DUP -1 <>                   \ End of table ?
    WHILE   
        OPT1                            \ No, optimiser enabled ?
        IF   COMP-ASS?                  \ Yes, assembler strings equal ?
             IF   C@+                   \ Yes, is it a macro type ?
                  IF   REMOVE-CODE      \ Yes, remove old code-string
                       ALIGNED @ EXECUTE \ Execute macro (lay down code)
                  ELSE ALIGNED @        \ Get new token and
                       OPT1 COMPILE!    \ Store over the old code
                  THEN
                  REPL-OPTI             \ Set optimiser code pointer back.
                  R> DROP               \ Throw orig. token address away
                  EXIT                  \ Ready.
             ELSE 1+                    \ Step over type byte
             THEN
        ELSE DROP 1+                    \ Remove code-string-addres & step.
        THEN
        CELL+                           \ To next optimiser entry
    REPEAT
    2DROP  R> @ COMPILE,                \ Compile default token instead. 
    ;

FORTH DEFINITIONS

\G ( -1 c-addr1 type1 xt1 ... c-addrn typen xtn "name" -- )
\G Skip leading space delimiters. Parse name delimited by a space.
\G Create a definition for name with the execution semantics
\G defined below. Build an optimiser table from the data on the 
\G stack.
\G
\G name Execution: ( i*x -- j*x )
\G Execute the execution token that originally belongs to the
\G word name. The stack effects i*x and j*x are due to the token
\G executed.
\G
\G name Compiling: ( -- )
\G Compile the execution token originally belonging to name,
\G or compile or execute one of the entries from the created table.
: OPTIMISER                             \ Optimiser  Build a table driven optimiser
    PUSH >IN                            \ Save input pointer
    '                                   \ Get <xt> of "name"
    POP >IN                             \ Restore >IN
    CREATE  IMMEDIATE ( i*x "name" -- ) \ Create immediate optimiser entry
        ,                               \ Compile original token
        BEGIN   DUP -1 <>               \ Not at end of table ?
        WHILE   ROT   ,                 \ Yes, build optimiser table, optimise token
                SWAP C,                 \ Type code (1=macro,0=token)
                ALIGN ,                 \ Optimiser macro or token
        REPEAT  ,                       \ Compile end of table
    DOES>   OPTIMISE,                   \ Handle one optimiser table.
    ;

                    \ (* End of Source *) /
