\ ==============================================================
\ [TITLE]     ANS Forth HCC-language day TINGLE words set. 
\ [FILE]      TINGLE.FRT
\ [CATEGORY]  Example
\ [VSN]       1.20 
\ [CREATED]   January 18, 1994, Willem Ouwerkerk 
\ [LASTCHNGD] October 20, 1994, Willem Ouwerkerk 
\ [COPYRIGHT] Dutch Forth Users Group (C) 1994
\ [COMPILE]   Originally for the HCC-language day: jan. '93
\ [MODULE OF] 8051-ANS-Forth 
\ [USES]      ...
\ [AUTHOR]    Willem Ouwerkerk/Albert van der Horst
\ [PURPOSE]   HCC-language day hardware program demo.
\             Control of a tingle-tangle. This is a slightly
\             updated and enhanced version. Note that it uses
\             no assembly code, not even to access the I/O port.
\ ==============================================================

\ d3 is the bit by bit logical "and" of d1 with d2.
: 2AND      ( d1 d2 -- d3 )             \ Tingle
    >R SWAP R> AND >R  AND R>
    ;

\ d2 is the result of shifting d1 one bit towards the most-
\ significant bit, filling the vacated least-significant bit
\ with zero.
: D2*           ( d1 -- d2 )            \ Tingle
        1 LSHIFT >R
        DUP $8000 AND >R
        1 LSHIFT
        2R> 15 RSHIFT  OR
        ;

\ ==== Sheet 4: The printerport ================================

ASSEMBLER

\ Access I/O port P1 high level. See also: SFR .
P1: SFR PORT   ( -- byte )              \ Tingle
FORTH

\ ==== Sheet 5: Hardware representation ========================

\ Hold a copy from the status of output port bits of PORT  .
%11111111 VALUE LATCH  ( -- x )         \ Tingle

\ Set all corresponding high bits of the bit pattern x in LATCH .
: SET-BIT   ( x -- )                    \ Tingle 00001000
    LATCH                               \ DDDDDDDD
    OR                                  \ DDDD1DDD
    TO LATCH
    ;

\ Clear all corresponding high bits of the bit pattern x in LATCH .
: CLR-BIT   ( x -- )                    \ Tingle 00001000
    INVERT                              \ 11110111
    LATCH                               \ DDDDDDDD
    AND                                 \ DDDD0DDD
    TO LATCH
    ;

\ ==== Sheet 6: Clock data in a shiftregister ==================

\ Set the data bit in the latch high
: SET-DATA  ( -- )                      \ Tingle
    %01000000 SET-BIT
    ;

\ Set the data bit in the latch to zero
: CLR-DATA  ( -- )                      \ Tingle
    %01000000 CLR-BIT
    ;

\ Clock one bit into the TINGLE shift register
: CLOCK     ( -- )                      \ Tingle
    %00100000 CLR-BIT
    LATCH TO PORT 
    %00100000 SET-BIT
    LATCH TO PORT 
    ;

\ ==== Sheet 7: Couple and decouple the shiftregister ==========

\ Set TINGLE shift register contents to the outputs
: ACCEPT     ( -- )                     \ Tingle
    %00010000 SET-BIT
    LATCH TO PORT 
    %00010000 CLR-BIT
    LATCH TO PORT 
    ;

\ ==== Sheet 8: We will control a set of 32 tingle =============

\ Move all notes u notes upward (transpose).
0 VALUE #MOVE        ( -- u )            \ Tingle

\ Clock a 32-bits data word to the TINGLE
: SET-TINGLE   ( 32bits-data -- )      \ Tingle
    CLR-DATA
    #16 #MOVE -  0 
    ?DO     CLOCK
    LOOP
    #32 0
    DO      2DUP $80000000. 2AND OR 0<>
            IF      SET-DATA    
            ELSE    CLR-DATA
            THEN
            CLOCK
            D2*
    LOOP    
    2DROP                               \ Remove used data word
    CLR-DATA
    #MOVE  0 
    ?DO     CLOCK
    LOOP
    ACCEPT
    ;

\ ==== Sheet 9: How will a note sound right ====================

\G Hold pulse length for sounding a note, u is the pulse length.
 #10 VALUE PULSELENGTH   ( -- u )       \ Tingle

\G Hold length of a note to play, u is the note length.
#400 VALUE NOTELENGTH    ( -- u )       \ Tingle

\ ==== Sheet 10: We will define some notes in Forth's way ======

\G ( bitvalue "name" -- )
\G Skip leading space delimiters. Parse name delimited by a space.
\G Create a new definition for name with the execution semantics
\G below.
\G
\G name Execution: ( -- )
\G Play the note corresponding with bitvalue on the TINGLE.
\G PULSELENGTH is the hit time and NOTELENGTH is the sound
\G time of the played note.
: NOTE                                  \ Tingle
    CREATE  , ,
    DOES>   DUP BODY> >NAME .NAME
            2@ SET-TINGLE
            PULSELENGTH MS
            0. SET-TINGLE
            NOTELENGTH MS
            ;

\G Activate the note G0, the note will be played using the
\G PULSELENGTH for the hit time, and NOTELENGTH is the word for
\G the total sound time of the played note. The defined notes are 
\G from G0 to B1 , enough to play Father Jacob. See also: NOTE .
%00000000000000001. NOTE G0  ( -- )     \ Tingle

%00000000000000100. NOTE A0
%00000000000010000. NOTE B0
%00000000000100000. NOTE C1
%00000000010000000. NOTE D1
%00000001000000000. NOTE E1
%00000010000000000. NOTE F1
%00001000000000000. NOTE G1
%00100000000000000. NOTE A1
%10000000000000000. NOTE B1

\G Transpose all notes +n half notes upwards with a maximum of 15.
: TRANSPOSE     ( +n -- )               \ Tingle
    $F AND TO #MOVE
    ;

\ ==== Sheet 11: Note length is also importand =================

\G Define the total sound time of a note. The defined sound times
\G are from 1/8 to 1/1 note length.
: 1/8       ( -- )                      \ Tingle
    #150 PULSELENGTH -  TO NOTELENGTH 
    ;
: 1/4       ( -- )                      \ Tingle 
    #300 PULSELENGTH -  TO NOTELENGTH 
    ;
: 1/2       ( -- )                      \ Tingle
    #600 PULSELENGTH -  TO NOTELENGTH 
    ;
: 1/1       ( -- )                      \ Tingle
    #1200 PULSELENGTH -  TO NOTELENGTH 
    ;

\ ==== Sheet 12: We build Father Jacob =========================

: LINE1     1/4  C1 D1 E1 C1 ;
: LINE2     1/4  E1 F1  1/2  G1 ;
: LINE3     1/8  G1 A1 G1 F1  1/4  E1 C1 ;
: LINE4     1/4  C1 G0  1/2  C1 ;

\G Play the Father Jacob tune on the TINGLE-TANGLE
: FATHER-JACOB   ( -- )                 \ Tingle
    LINE1  LINE1
    LINE2  LINE2
    LINE3  LINE3
    LINE4  LINE4
    ;

                    \ (* End of Source *) /
